%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: estimate_equidist_mutual_inf.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
% 		Estimate the mutual information between two time
%       series using the standard equidistant binning approach.
%
% Synopsis:
%
%	mi = estimate_equidist_mutual_inf(wph, bins)
%
% Arguments:
%
%   wph : the two time series (column vectors), can be series of any type
%		(phases or source signal)
%
%   bins : the number of bins into which the series are to be split
%
% Note:
%
%	This method does not apply any bias corrections.
%
% Returns:
%
%   mi : the mutual information of the time series estimated using
%		 the equiquantal method
%
% Examples:
%
%	None.
%
function mi = estimate_equidist_mutual_inf(wph, bins)

	% length of input time series
	N = size(wph, 1);

	% prepare accumulation buffers
	bins_x = zeros(1, bins);
	bins_y = zeros(1, bins);
	bins_xy = zeros(1, bins * bins);

	% compute bin indices for samples
	bin_wdth = 2*pi / bins;
	ndx_x = floor( wph(:,1) / bin_wdth) + 1;
	ndx_y = floor( wph(:,2) / bin_wdth) + 1;
	
	% accumulate all bin buffers
	for i=1:N

		bins_x(ndx_x(i)) = bins_x(ndx_x(i)) + 1;
		bins_y(ndx_y(i)) = bins_y(ndx_y(i)) + 1;

		ndx_xy = (ndx_x(i) - 1) * bins + ndx_y(i);
		bins_xy(ndx_xy) = bins_xy(ndx_xy) + 1;

	end

	% normalize to probabilities
	bins_x = bins_x / N;
	bins_y = bins_y / N;
	bins_xy = bins_xy / N;

	% compute marginal entropies and joint entropy
	hx = shannon_entropy(bins_x);
	hy = shannon_entropy(bins_y);
	hxy = shannon_entropy(bins_xy);

	% mutual information
	mi = hx + hy - hxy;
